#include "stdafx.h"
#include <stdio.h>
#include "Skate.h"

#include "Ogre.h"
#include "PhysXHandler.h"

using namespace Ogre;

#ifndef H_AI
#define H_AI


struct Rule {

	double dmin;
	double dmax;
	double angmin;
	double angmax;

	double speed;
	double turn;
	bool shoot;
};

class AI {

	std::vector<Rule*> rules;
	Skate* skate;
	Rule* defRule;

	PhysicsHandler* ph;
	SceneManager* sm;

public:

	AI(const char* fileName, Skate* skate, SceneManager* sm, PhysicsHandler* ph) {
		this->skate = skate;
		this->ph = ph;
		this->sm = sm;
		std::string line;
		std::string word;
		std::ifstream fp_in; 
		fp_in.open(fileName, std::ios::in);    
		getline (fp_in,line);
		std::istringstream iss(line);
		
		int ruleNum;
		iss>>ruleNum;

		
		getline (fp_in,line);
		std::istringstream iss2(line);
		
		defRule = new Rule();
		iss2 >> defRule->speed;
		iss2 >> defRule->shoot;
		iss2 >> defRule->turn;

		for(int i=0;i<ruleNum;i++) {
			getline (fp_in,line);
			std::istringstream iss3(line);
			Rule* rule = new Rule();
			iss3>>rule->dmin;
			iss3>>rule->dmax;
			iss3>>rule->angmin;
			iss3>>rule->angmax;
			
			iss3>>rule->speed;
			iss3>>rule->turn;
			iss3>>rule->shoot;

			rules.push_back(rule);
			
		}

	}

	void shoot() {
		
		SceneNode* ballNode = sm->getRootSceneNode()->createChildSceneNode();
		if(skate->shoot(ballNode)) {
			NxQuat dir = skate->getActor()->getGlobalOrientationQuat();
			ph->addShot(ballNode, dir);
		}
		
	}

	void fire(double dist, double ang) {
		for(unsigned int i=0;i<rules.size();i++) {
			Rule* rule = rules[i];
		/*	std::cout<<i<<"  ";
			std::cout << rule->angmin << " " << rule->angmax << " " << ang <<"\n";
			std::cout << rule->dmin << " " << rule->dmax << " " << dist<<"\n";*/
			if(ang >= rule->angmin && ang <= rule->angmax && dist >= rule->dmin && dist <= rule->dmax) {
				//std::cout<<"\n\n"<<rule->speed<<" "<<rule->turn<<" "<<rule->shoot<<"\n\n";
				skate->move(rule->speed);
				skate->turn(rule->turn);
				if(rule->shoot) {
					shoot();
				}
				return;
			}
		}
		if(defRule != NULL) {
			//std::cout<<"\n\n def "<<defRule->speed<<" "<<defRule->turn<<" "<<defRule->shoot<<"\n\n";
			skate->move(defRule->speed);
			skate->turn(defRule->turn);
			if(defRule->shoot) {
				shoot();
			}
		}
	}

};

#endif
