#include "stdafx.h"

#include "Ogre.h"
#include "InputHandler.h"
#include "PhysXHandler.h"
#include "NxPhysics.h"
#include "Level.h"
#include "Skate.h"
#include "ThirdPersonCamera.h "
#include "PlayerController.h"
#include "Particles.h"
#include "AISkate.h"

using namespace Ogre;

Root* ogreRoot;
SceneManager* sceneManager;
Camera* camera;
RenderWindow* renderWindow;
InputHandler* inputHandler;
PhysicsHandler* physxHandler;
Skate* skate;
ThirdPersonCamera* tpCamera; 
PlayerController* playerController;
std::vector<AISkate*> aiskates;
Billboard* bb;

bool gameEnded;

static const int AI_NUM=5;

void win(bool win) {
	
	SceneNode* billboardNode = sceneManager->getRootSceneNode()->createChildSceneNode();
	BillboardSet* winBillboardSet = sceneManager->createBillboardSet(win?"winbb":"loosebb", 1);
	billboardNode->attachObject(winBillboardSet);
	
	winBillboardSet->setDefaultDimensions(30,5);
	winBillboardSet->setBillboardType(BBT_ORIENTED_COMMON);
	winBillboardSet->setBillboardOrigin(BBO_BOTTOM_CENTER);
	winBillboardSet->setCommonDirection(Vector3::UNIT_Y);
	Vector3 pos = skate->getPosition();
	double ar = camera->getOrientation().getYaw().valueRadians();
	pos.y = pos.y + 5;
	pos.x = pos.x - sin(ar)*50;
	pos.z = pos.z - cos(ar)*50;
	
	bb = winBillboardSet->createBillboard(pos);
	
	if(win) {
		winBillboardSet->setMaterialName("WinBillboard");
	} else {
		winBillboardSet->setMaterialName("LooseBillboard");
	}
	gameEnded = true;
		
	
}

class mainFrameListener : public FrameListener
{
protected:
	float t;
public:
	mainFrameListener()
	{
		t = 0;
	}

	bool frameStarted(const FrameEvent& evt)
	{
		float dt = evt.timeSinceLastFrame;
		t += dt;
		if(!inputHandler->update(t, dt))
			return false;
		if(!physxHandler->update(t, dt))
			return false;
		skate->update(t, dt);
		if(bb != NULL) {
			Vector3 pos = skate->getPosition();
			double ar = camera->getOrientation().getYaw().valueRadians();
			
			pos.y = pos.y + 5;
			pos.x = pos.x - sin(ar)*50;
			pos.z = pos.z - cos(ar)*50;
			//std::cout<<ar<<" "<<pos.x<<" "<<pos.z<<"\n";
			bb->setPosition(pos);
		}

		for(unsigned int i=0;i<AI_NUM;i++) {
			if(!aiskates[i]->update(t, dt))
			return false;
		}
		tpCamera->update(t, dt);

		if(!gameEnded) {
			if(!skate->isAlive()) {
				win(false);
			} else {
				bool otherSkateAlive = false;
				for(unsigned int i=0;i<AI_NUM;i++) {
					if(aiskates[i]->getSkate()->isAlive()) {
						otherSkateAlive = true;
						break;
					}
				}
				if(!otherSkateAlive) {
					win(true);
				}
			}
		}
	
		return true;
	}
};




void init()
{
	gameEnded = false;
	bb=NULL;
	std::vector<Skate*>* targets = new std::vector<Skate*>();
	inputHandler = new InputHandler(renderWindow);
	physxHandler = new PhysicsHandler(sceneManager, targets);
	
	camera = sceneManager->createCamera("Main Camera");       
	camera->setPosition(20, 20, 20); 
	camera->lookAt(0,5,0);
	camera->setFarClipDistance(1000);
	camera->setNearClipDistance(0.1);

	Viewport* viewport = renderWindow->addViewport(camera);
	renderWindow->setActive(true); 
	skate = new Skate(sceneManager, Vector3(0,2,0), Vector3(1, 0, 0), "player", 60);
	NxActor* skateActor = physxHandler->addSkate(skate->getSceneNode(), skate->getSmoker()->getSceneNode());
	skate->setActor(skateActor);
	targets->push_back(skate);

	tpCamera = new ThirdPersonCamera();
	tpCamera->setCamera(camera);
	tpCamera->setCharacter(skate);

	new Level(sceneManager, physxHandler);

	playerController = new PlayerController(skate, sceneManager, physxHandler);
	inputHandler->addPlayerController(playerController);

	ogreRoot->addFrameListener(new mainFrameListener());
	ogreRoot->addFrameListener(physxHandler);

	
	for(int i=0;i<AI_NUM;i++) {
		
		double angle = Math::RangeRandom(0, 6.28);
		AISkate* aiskate = new AISkate(sceneManager, physxHandler, 
			Vector3(Math::RangeRandom(0, 300)-150, 2, Math::RangeRandom(0, 300)-150),
			Vector3(cos(angle), 0, sin(angle)),
			"aiplayer"+StringConverter::toString(i));
		ogreRoot->addFrameListener(aiskate);
		aiskates.push_back(aiskate);
		targets->push_back(aiskate->getSkate());
		aiskate->addTarget(skate);
	}

	for(int i=0;i<AI_NUM;i++) {
		for(int j=0;j<AI_NUM;j++) {
			if(i != j) {
				aiskates[i]->addTarget(aiskates[j]->getSkate());
			}
		}
	}

	
	

}

int _tmain(int argc, _TCHAR* argv[])
{
	ogreRoot = new Root("","");
#ifdef _DEBUG
	ogreRoot->loadPlugin("RenderSystem_Direct3D9_d");
	ogreRoot->loadPlugin("Plugin_OctreeSceneManager_d");
	ogreRoot->loadPlugin("Plugin_ParticleFX_d");

#else
	ogreRoot->loadPlugin("RenderSystem_Direct3D9");
	ogreRoot->loadPlugin("Plugin_OctreeSceneManager");
	ogreRoot->loadPlugin("Plugin_ParticleFX");

#endif

	const RenderSystemList& list = ogreRoot->getAvailableRenderers(); 
	ogreRoot->setRenderSystem(list.at(0));
	ogreRoot->initialise(false);
	renderWindow = ogreRoot->createRenderWindow("GameDev Ogre Player Character", 800, 600, false);

	sceneManager = ogreRoot->createSceneManager("OctreeSceneManager"); 

	ResourceGroupManager::getSingleton().addResourceLocation("media","FileSystem","General", false);
	ResourceGroupManager::getSingleton().initialiseAllResourceGroups(); 

	
	init();
	
	//ogreRoot->startRendering(); 
	while(true)
	{
		if(renderWindow->isClosed())
		{
			ogreRoot->shutdown();
			break;
		}
		WindowEventUtilities::messagePump();
		
		if(!ogreRoot->renderOneFrame())
		{
			ogreRoot->shutdown();
			break;
		}
	}

	return 0;
}