#pragma once
#define OIS_DYNAMIC_LIB
#include <OIS/OIS.h>
#include "Ogre.h"
#include "PlayerController.h"

class InputHandler : public Ogre::FrameListener
{
protected:
	OIS::InputManager* OISInputManager;
	OIS::Keyboard* mKeyboard;
	OIS::Mouse* mMouse;
	float mTimeUntilNextToggle;
	std::list<PlayerController*> controllers;

public:
	InputHandler(Ogre::RenderWindow* renderWindow)
	{
		Ogre::LogManager::getSingletonPtr()->logMessage("*** Initializing OIS ***");
		
		OIS::ParamList pl;

		size_t windowHnd = 0;
		renderWindow->getCustomAttribute("WINDOW", &windowHnd);
		std::ostringstream windowHndStr;
		windowHndStr << windowHnd;
		pl.insert(std::make_pair(std::string("WINDOW"), windowHndStr.str()));
		OISInputManager = OIS::InputManager::createInputSystem( pl );

		mKeyboard = static_cast<OIS::Keyboard*>(OISInputManager->createInputObject( OIS::OISKeyboard, false ));
		mMouse = static_cast<OIS::Mouse*>(OISInputManager->createInputObject( OIS::OISMouse, false ));
		
	}
	~InputHandler(void)
	{
		Ogre::LogManager::getSingletonPtr()->logMessage("*** Shutting down OIS ***");
		OIS::InputManager::destroyInputSystem(OISInputManager);
	}

	bool update(float t, float dt)
	{
		mKeyboard->capture();
		mMouse->capture();
		
		const OIS::MouseState &ms = mMouse->getMouseState();
		
		if (mTimeUntilNextToggle >= 0)
			mTimeUntilNextToggle -= dt;
		
		if( mKeyboard->isKeyDown(OIS::KC_ESCAPE))
			return false;
		
		std::list<PlayerController*>::iterator it = controllers.begin();
		std::list<PlayerController*>::iterator itend = controllers.end();
		while(it != itend)
		{
			(*it)->update(t, dt, mKeyboard, mMouse);
			it++;
		}
		return true;
	}

	void addPlayerController(PlayerController* c)
	{
		controllers.push_back(c);
	}


};
