#include "StdAfx.h"
#include "Level.h"
#include "Tree.h"
#include "PhysxHandler.h"
#include <fstream>
#include <iostream>

using namespace std;
using namespace Ogre;

ManualObject* Level::createCubeMesh(Ogre::String name, Ogre::String matName) {

   ManualObject* cube = new ManualObject(name);
   cube->begin(matName);
   
   cube->position(0.5,-0.5,0.5);cube->normal(0.408248,-0.816497,0.408248);cube->textureCoord(1,0);
   cube->position(-0.5,-0.5,-0.5);cube->normal(-0.408248,-0.816497,-0.408248);cube->textureCoord(0,1);
   cube->position(0.5,-0.5,-0.5);cube->normal(0.666667,-0.333333,-0.666667);cube->textureCoord(1,1);
   cube->position(-0.5,-0.5,0.5);cube->normal(-0.666667,-0.333333,0.666667);cube->textureCoord(0,0);
   cube->position(0.5,0.5,0.5);cube->normal(0.666667,0.333333,0.666667);cube->textureCoord(1,0);
   cube->position(-0.5,-0.5,0.5);cube->normal(-0.666667,-0.333333,0.666667);cube->textureCoord(0,1);
   cube->position(0.5,-0.5,0.5);cube->normal(0.408248,-0.816497,0.408248);cube->textureCoord(1,1);
   cube->position(-0.5,0.5,0.5);cube->normal(-0.408248,0.816497,0.408248);cube->textureCoord(0,0);
   cube->position(-0.5,0.5,-0.5);cube->normal(-0.666667,0.333333,-0.666667);cube->textureCoord(0,1);
   cube->position(-0.5,-0.5,-0.5);cube->normal(-0.408248,-0.816497,-0.408248);cube->textureCoord(1,1);
   cube->position(-0.5,-0.5,0.5);cube->normal(-0.666667,-0.333333,0.666667);cube->textureCoord(1,0);
   cube->position(0.5,-0.5,-0.5);cube->normal(0.666667,-0.333333,-0.666667);cube->textureCoord(0,1);
   cube->position(0.5,0.5,-0.5);cube->normal(0.408248,0.816497,-0.408248);cube->textureCoord(1,1);
   cube->position(0.5,-0.5,0.5);cube->normal(0.408248,-0.816497,0.408248);cube->textureCoord(0,0);
   cube->position(0.5,-0.5,-0.5);cube->normal(0.666667,-0.333333,-0.666667);cube->textureCoord(1,0);
   cube->position(-0.5,-0.5,-0.5);cube->normal(-0.408248,-0.816497,-0.408248);cube->textureCoord(0,0);
   cube->position(-0.5,0.5,0.5);cube->normal(-0.408248,0.816497,0.408248);cube->textureCoord(1,0);
   cube->position(0.5,0.5,-0.5);cube->normal(0.408248,0.816497,-0.408248);cube->textureCoord(0,1);
   cube->position(-0.5,0.5,-0.5);cube->normal(-0.666667,0.333333,-0.666667);cube->textureCoord(1,1);
   cube->position(0.5,0.5,0.5);cube->normal(0.666667,0.333333,0.666667);cube->textureCoord(0,0);

   cube->triangle(0,1,2);      cube->triangle(3,1,0);
   cube->triangle(4,5,6);      cube->triangle(4,7,5);
   cube->triangle(8,9,10);      cube->triangle(10,7,8);
   cube->triangle(4,11,12);   cube->triangle(4,13,11);
   cube->triangle(14,8,12);   cube->triangle(14,15,8);
   cube->triangle(16,17,18);   cube->triangle(16,19,17);
   cube->end(); 

   return cube;

}

void Level::createHouse(SceneManager* sm, PhysicsHandler* ph, double px, double pz, int id) {
	Entity* house = sm->createEntity("house"+Ogre::StringConverter::toString(id), "house.mesh");
	SceneNode* houseNode = sm->getRootSceneNode()->createChildSceneNode();
	houseNode->attachObject(house);
	houseNode->setPosition(px, 15, pz);
	houseNode->setScale(10, 7, 10);

	ph->addBox(houseNode, true, true, Vector3(8,20,10), Vector3(10,0,0));
}

void Level::createGround(SceneManager* sm)
{
	MovablePlane* plane = new MovablePlane("GroundPlane");
        plane->d = 0;
        plane->normal = Vector3::UNIT_Y;
		MeshManager::getSingleton().createPlane(
			"GroundPlane", 
			ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
           			*plane,
			500,// x-size
			500,// y-size
           			1,	// x-segments
			1,  // y-segments
			true,
			1,
			10, // u-tile
			10, // v-tile
			Vector3::UNIT_Z);
		Entity* planeEnt = sm->createEntity( "GroundPlane", "GroundPlane" );
		planeEnt->setMaterialName("Ground");
		planeEnt->setCastShadows(false);
		sm->getRootSceneNode()->createChildSceneNode()->attachObject(planeEnt);
}

void Level::createLight(SceneManager* sm) {
	Light* light;
	light = sm->createLight("pointlight1");
	light->setType(Light::LT_POINT);
	light->setAttenuation(1000,0,0,1);
	light->setPowerScale(1000);
	light->setCastShadows(true);
	light->setPosition(500, 100, 0);
}

void Level::createFire(SceneManager* sm, double px, double pz, int id) {

	Entity* fire = sm->createEntity("fire"+Ogre::StringConverter::toString(id), "campfire.mesh");
	SceneNode* fireNode = sm->getRootSceneNode()->createChildSceneNode();
	fireNode->attachObject(fire);
	fireNode->setPosition(px, 0.5, pz);
	fireNode->setScale(0.5, 0.5, 0.5);
	
	new PSysFromScript(Vector3(px,0,pz),sm,"Campfire");
}

void Level::createHill(SceneManager* sm, PhysicsHandler* ph, double s, double angle1, double angle2, double px, double pz, int id) {
	double pi4 = Ogre::Math::PI / 4.0;
	
	SceneNode* box1Node = sm->getRootSceneNode()->createChildSceneNode();
	box1Node->attachObject(createCubeMesh("box1"+Ogre::StringConverter::toString(id), "Snow")); 
	box1Node->setScale(s,s,s);
	box1Node->setPosition(px+(s/2)*sin(angle2)*cos(angle1+pi4)*1.414,-s/2,pz+(s/2)*cos(angle2)*cos(angle1+pi4)*1.414);
	box1Node->rotate(Ogre::Vector3::UNIT_X, Ogre::Radian(angle1), Ogre::Node::TS_WORLD);
	box1Node->rotate(Ogre::Vector3::UNIT_Y, Ogre::Radian(angle2), Ogre::Node::TS_WORLD);
	
	ph->addBox(box1Node, true, false, Vector3(0,0,0), Vector3(0,0,0));

	SceneNode* box2Node = sm->getRootSceneNode()->createChildSceneNode();
	box2Node->attachObject(createCubeMesh("box2"+Ogre::StringConverter::toString(id), "Snow")); 
	box2Node->setScale(s,s,s);
	box2Node->setPosition(px-(s/2)*sin(angle2)*cos(angle1+pi4)*1.414,-s/2,pz-(s/2)*cos(angle2)*cos(angle1+pi4)*1.414);
	box2Node->rotate(Ogre::Vector3::UNIT_X, Ogre::Radian(-angle1), Ogre::Node::TS_WORLD);
	box2Node->rotate(Ogre::Vector3::UNIT_Y, Ogre::Radian(angle2), Ogre::Node::TS_WORLD);
	
	ph->addBox(box2Node, true, false, Vector3(0,0,0), Vector3(0,0,0));
	
}

void Level::loadLevel(const char* fileName, SceneManager* sm, PhysicsHandler* physxHandler) {
	std::string line;
	std::string word;
	std::ifstream fp_in; 
	fp_in.open(fileName, std::ios::in);    
	getline (fp_in,line);
	std::istringstream iss(line);
		
	int objectNum;
	iss>>objectNum;


	for(unsigned int i=0;i<objectNum;i++) {
		getline(fp_in,line);
		std::istringstream iss2(line);
		std::string typ;
		iss2>>typ;
		if(typ.compare("campfire") == 0) {
			double px;
			double pz;
			iss2>>px;
			iss2>>pz;
			createFire(sm, px, pz, i);
		} else if(typ.compare("hill") == 0){
			double s;
			double a1;
			double a2;
			double px;
			double pz;
			iss2>>s;
			iss2>>a1;
			iss2>>a2;
			iss2>>px;
			iss2>>pz;
			createHill(sm, physxHandler, s, a1, a2, px, pz, i);
		} else if(typ.compare("tree") == 0){
			double px;
			double pz;
			iss2>>px;
			iss2>>pz;
			Vector3 pos = Vector3(px, 0, pz);
			BillBoardCloudTree* tree = new BillBoardCloudTree(pos, sm);
			physxHandler->addBox(tree->getSceneNode(), false, false, Vector3(0,0,0), Vector3(0,0,0));
		} else if(typ.compare("house") == 0){
			double px;
			double pz;
			iss2>>px;
			iss2>>pz;
			createHouse(sm, physxHandler, px, pz, i);
		}
	}
}

void Level::createWall(SceneManager* sm, double px, double pz, double sx, double sz, const char* mat, int id) {
	SceneNode* box1Node = sm->getRootSceneNode()->createChildSceneNode();
	box1Node->attachObject(createCubeMesh("wall"+Ogre::StringConverter::toString(id),mat)); 
	box1Node->setScale(sx,5,sz);
	box1Node->setPosition(px, 0, pz);
}

Level::Level(SceneManager* sm, PhysicsHandler* physxHandler)
{
	NxScene* nxs = physxHandler->getScene();
	SceneNode* rootNode = sm->getRootSceneNode();
	SceneNode* levelNode = rootNode->createChildSceneNode();
	
	Light * mainLight = sm->createLight("mainLight");
	mainLight->setType(Light::LT_DIRECTIONAL);
	mainLight->setDirection(-1,-1,-1);

	sm->setSkyBox(true, "Sky", 10, true);	

	createGround(sm);

	createWall(sm, -250, 0, 1, 500, "SnowWall1", 1);
	createWall(sm, 250, 0, 1, 500, "SnowWall1", 2);
	createWall(sm, 0, 250, 500, 1, "SnowWall1", 3);
	createWall(sm, 0, -250, 500, 1, "SnowWall1", 4);


	loadLevel("level1.level", sm, physxHandler);
}

Level::~Level(void)
{
}

