#include "stdafx.h"
#include "PhysXHandler.h"

PhysXSkateObject::PhysXSkateObject(NxScene* physXScene, SceneNode* _sn, SceneNode* _smokerNode)
{
		a=0;
		ogreNode = _sn;
		smokerNode = _smokerNode;
		NxMaterialDesc material;
		material.restitution =	0;
		material.dynamicFriction = 0.1;
		material.staticFriction = 0.1;
		material.restitutionCombineMode = NX_CM_MIN;
		material.frictionCombineMode = NX_CM_MAX;
		NxMaterial* mat = physXScene->createMaterial( material);
		NxBoxShapeDesc boxDesc;
		boxDesc.materialIndex = mat->getMaterialIndex();
		mat = physXScene->getMaterialFromIndex(0);
		mat->setFrictionCombineMode(NX_CM_MAX);
		mat->setRestitutionCombineMode(NX_CM_MIN);
		
		boxDesc.dimensions.set(0.7, 0.8, 2);	
		boxDesc.localPose.t.set(0.9,0,1);
		actorDesc.shapes.pushBack(&boxDesc);
		
		actorDesc.body = &bodyDesc;
		actorDesc.density = 0.6;
		actorDesc.globalPose.t 	= NxVec3(
			ogreNode->getPosition().x,
			ogreNode->getPosition().y,
			ogreNode->getPosition().z);
		actor = physXScene->createActor(actorDesc);
		NxQuat q;
		q.setw(ogreNode->getOrientation().w);
		q.setx(ogreNode->getOrientation().x);
		q.sety(ogreNode->getOrientation().y);
		q.setz(ogreNode->getOrientation().z);
		actor->setGlobalOrientationQuat(q);
		
}

void PhysXSkateObject::update()
{
		ogreNode->resetOrientation();
		
		NxQuat q = actor->getGlobalOrientationQuat();
		ogreNode->setOrientation(q.w,q.x,q.y,q.z);

		ogreNode->setPosition(actor->getGlobalPosition().x,
								actor->getGlobalPosition().y,
								actor->getGlobalPosition().z);
		
		double ar = ogreNode->getOrientation().getYaw().valueRadians();
		smokerNode->setPosition(actor->getGlobalPosition().x+cos(ar),
								actor->getGlobalPosition().y,
								actor->getGlobalPosition().z-sin(ar));
		
}

PhysXBoxObject::PhysXBoxObject(SceneNode* ogreNode, NxScene* physXScene, bool direct, bool rePos,  Vector3 scale, Vector3 relPos) 
{
		this->ogreNode = ogreNode;
		NxMaterialDesc material;
		material.restitution = 0.01;
		material.dynamicFriction = 0.01;
		material.staticFriction = 0.01;
		material.restitutionCombineMode = NX_CM_MIN;
		material.frictionCombineMode = NX_CM_MAX;
		NxMaterial* mat = physXScene->createMaterial( material);
		boxDesc.materialIndex = mat->getMaterialIndex();
		mat = physXScene->getMaterialFromIndex(0);
		mat->setFrictionCombineMode(NX_CM_MAX);
		mat->setRestitutionCombineMode(NX_CM_MIN);
		
		if(!rePos) {
			boxDesc.dimensions.set(ogreNode->getScale().x*0.5,
				ogreNode->getScale().y*0.5,
				ogreNode->getScale().z*0.5);	
		} else {
			boxDesc.dimensions.set(scale.x, scale.y, scale.z);
		}
		
		actorDesc.shapes.pushBack(&boxDesc);
		actorDesc.density = 1;
		if(!rePos) {
			actorDesc.globalPose.t 	= NxVec3(ogreNode->getPosition().x,
				ogreNode->getPosition().y,
				ogreNode->getPosition().z);
		} else {
			actorDesc.globalPose.t 	= NxVec3(ogreNode->getPosition().x + relPos.x,
				ogreNode->getPosition().y + relPos.y,
				ogreNode->getPosition().z + relPos.z);
		}
		
		actor = physXScene->createActor(actorDesc);
		actorDesc.body = 0;
		if(direct) {
			NxQuat q;
			q.setw(ogreNode->getOrientation().w);
			q.setx(ogreNode->getOrientation().x);
			q.sety(ogreNode->getOrientation().y);
			q.setz(ogreNode->getOrientation().z);
			actor->setGlobalOrientationQuat(q);
		}		
}	


PhysXBallObject::PhysXBallObject(SceneNode* ogreNode, NxScene* physXScene, double dens, NxQuat dir, SceneManager* sceneManager, std::vector<Skate*>* targets)
{
		this->ogreNode = ogreNode;
		this->physXScene = physXScene;
		this->sceneManager = sceneManager;
		this->targets = targets;

		ucnt=0;
		NxMaterialDesc material;
		material.restitution = 0;
		material.dynamicFriction = 1;
		material.staticFriction = 1;
		material.restitutionCombineMode = NX_CM_MIN;
		material.frictionCombineMode = NX_CM_MAX;
		NxMaterial* mat = physXScene->createMaterial( material);
		sphereDesc.materialIndex = mat->getMaterialIndex();
		mat = physXScene->getMaterialFromIndex(0);
		mat->setFrictionCombineMode(NX_CM_MAX);
		mat->setRestitutionCombineMode(NX_CM_MIN);
		
		sphereDesc.radius = 0.3;	
		actorDesc.shapes.pushBack(&sphereDesc);
		actorDesc.body = &bodyDesc;
		actorDesc.density = dens;
		actorDesc.globalPose.t 	= NxVec3(
			ogreNode->getPosition().x,
			ogreNode->getPosition().y,
			ogreNode->getPosition().z);
		
		actor = physXScene->createActor(actorDesc);
		actor->setGlobalOrientationQuat(dir);
		actor->addLocalForce(NxVec3(0,15,80), NX_FORCE);
		active = true;
}

bool PhysXBallObject::update()
{
		if(active) {
			//Not exploded yet
			ogreNode->setOrientation(actor->getGlobalOrientationQuat().w,
									actor->getGlobalOrientationQuat().x,
									actor->getGlobalOrientationQuat().y,
									actor->getGlobalOrientationQuat().z);
			ogreNode->setPosition(actor->getGlobalPosition().x,
									actor->getGlobalPosition().y,
									actor->getGlobalPosition().z);

			double v = actor->getLinearVelocity().magnitude();
			if(v<100 && ucnt > 5) {
				//Explosion
				active = false;

				explosion = new PSysFromScript(Vector3(
					actor->getGlobalPosition().x,
					actor->getGlobalPosition().y,
					actor->getGlobalPosition().z),sceneManager,"Expl");

				//Damage skates
				for(unsigned int i=0;i<targets->size();i++) {
					Skate* tskate = (*targets)[i];
					if(tskate->isAlive()) {
						double dist = (tskate->getActor()->getGlobalPosition() - actor->getGlobalPosition()).magnitude();
						if(dist < 30) {
							tskate->damage((30 - dist) * 2);
						}
					}
				}

				sceneManager->getRootSceneNode()->removeChild(ogreNode->getName());
				physXScene->releaseActor(*actor);
				ucnt=6;
				return true;

			} else {
				//Flying
				ucnt++;
				return false;
			}
		} else if(ucnt > 5){
			//Exploded
			if(ucnt < 150) {
				//Explosion anim
				ucnt++;
				explosion->updateER(-5);
			} else if(ucnt==150){
				sceneManager->getRootSceneNode()->removeChild(explosion->getSceneNode()->getName());
				ucnt++;
			}
		}
}

void PhysicsHandler::createWall(double sx, double sy, double sz, double px, double py, double pz) {
		NxActorDesc actorDesc;				
		NxBoxShapeDesc boxDesc;
		NxActor* actor;
		
		boxDesc.materialIndex = 0;
		boxDesc.dimensions.set(sx, sy, sz);	
		
		actorDesc.shapes.pushBack(&boxDesc);
		actorDesc.density = 1;
		actorDesc.globalPose.t 	= NxVec3(px, py, pz);		
		actor = mPhysicsScene->createActor(actorDesc);
		actorDesc.body = 0;
		
}

PhysicsHandler::PhysicsHandler(SceneManager* sceneManager, std::vector<Skate*>* targets)
{
		inited = false;
		this->sceneManager = sceneManager;
		this->targets = targets;
		mPhysicsSDK = NxCreatePhysicsSDK(NX_PHYSICS_SDK_VERSION);
		if (!mPhysicsSDK)  return;

		mPhysicsSDK->getFoundationSDK().getRemoteDebugger()->connect("localhost", 5425);

		mPhysicsSDK->setParameter(NX_SKIN_WIDTH, 0.01);
		NxSceneDesc sceneDesc;
 		sceneDesc.simType = NX_SIMULATION_HW;
		sceneDesc.gravity = NxVec3(0,-90.8f,0);
		sceneDesc.groundPlane = true;
		
		mPhysicsScene = mPhysicsSDK->createScene(sceneDesc);	
		if(!mPhysicsScene)
		{ 
			Ogre::LogManager::getSingleton().logMessage("Harsware physX simulation not supported, switching to software");
		
			sceneDesc.simType = NX_SIMULATION_SW; 
			mPhysicsScene = mPhysicsSDK->createScene(sceneDesc);  
			if(!mPhysicsScene) return;
		}
		
		

		
		// Create the default material
		NxMaterial* defaultMaterial = mPhysicsScene->getMaterialFromIndex(0); 
		defaultMaterial->setRestitution(0.01);
		defaultMaterial->setStaticFriction(0.01);
		defaultMaterial->setDynamicFriction(0.01);
	
		createWall(1, 10, 500, 250, 0, 0);
		createWall(1, 10, 500, -250, 0, 0);
		createWall(500, 10, 1, 0, 0, 250);
		createWall(500, 10, 1, 0, 0, -250);

		inited = true;
}

bool PhysicsHandler::update(float t, float dt)
	{
		if(!inited)
			return true;

		mPhysicsScene->simulate(dt);
		mPhysicsScene->flushStream();
		while (!mPhysicsScene->fetchResults(NX_RIGID_BODY_FINISHED, false));

		for(unsigned int i=0;i<balls.size();i++) {
			balls[i]->update();
		}
		for(unsigned int i=0;i<skates.size();i++) {
			skates[i]->update();
		}
		
		return true;
	}