#include "stdafx.h"
#include "NxPhysics.h"
#include "Ogre.h"
#include "Particles.h"
#include "Skate.h"

using namespace Ogre;

#ifndef H_PhysXBoxObject
#define H_PhysXBoxObject

class Skate;


class PhysXSkateObject
{
	SceneNode* ogreNode;
	NxActorDesc actorDesc;				
	NxActor* actor;
	NxBodyDesc bodyDesc;
	SceneNode* smokerNode;

	double a;
public:
	PhysXSkateObject(NxScene* physXScene, SceneNode* _sn, SceneNode* _smokerNode);
	
	void update();
	
	NxActor* getActor() {
		return actor;
	}
};

class PhysXBoxObject
{
protected:
	SceneNode* ogreNode;
	NxActorDesc actorDesc;				
	NxBoxShapeDesc boxDesc;
	NxActor* actor;
	NxBodyDesc bodyDesc;
	
	
public:
	PhysXBoxObject(SceneNode* ogreNode, NxScene* physXScene, bool direct, bool rePos,  Vector3 scale, Vector3 relPos);
	

};



class PhysXBallObject
{
protected:
	SceneNode* ogreNode;
	NxScene* physXScene;
	NxActorDesc actorDesc;				
	NxSphereShapeDesc sphereDesc;
	NxActor* actor;
	NxBodyDesc bodyDesc;
	SceneManager* sceneManager;

	std::vector<Skate*>* targets;

	bool active;
	int ucnt;

	PSysFromScript* explosion;

	
public:
	
	PhysXBallObject(SceneNode* ogreNode, NxScene* physXScene, double dens, NxQuat dir, SceneManager* sceneManager, std::vector<Skate*>* targets);
	
	bool update();
	

	NxActor* getActor() {
		return actor;
	}
};


class PhysicsHandler : public FrameListener
{
protected:
	bool inited;
	NxPhysicsSDK* mPhysicsSDK;
	NxScene* mPhysicsScene;
	
	std::vector<PhysXBoxObject*> boxes;
	std::vector<PhysXBallObject*> balls;
	std::vector<PhysXSkateObject*> skates;
	SceneManager* sceneManager;

	std::vector<Skate*>* targets;

	void createWall(double sx, double sy, double sz, double px, double py, double pz);


public:
	NxScene* getScene(){return mPhysicsScene;}

	PhysicsHandler(SceneManager* sceneManager, std::vector<Skate*>* targets);

	NxActor* addSkate(SceneNode* _skate, SceneNode* _smoker) {
		PhysXSkateObject* skate = new PhysXSkateObject(mPhysicsScene, _skate, _smoker);
		skates.push_back(skate);
		return skate->getActor();
	}

	void addBox(SceneNode* ogreNode, bool direct, bool rePos,  Vector3 scale, Vector3 relPos) {
		boxes.push_back(new PhysXBoxObject(ogreNode, mPhysicsScene, direct, rePos, scale, relPos));
	}
	
	bool update(float t, float dt);


	void addShot(SceneNode* ogreNode, NxQuat dir)
	{
		PhysXBallObject* pbo = new PhysXBallObject(ogreNode, mPhysicsScene, 0.1, dir, sceneManager, targets);
		balls.push_back(pbo);
		
	}

};

#endif