#include "stdafx.h"
#include "Skate.h"

Skate::Skate(Ogre::SceneManager* sm, Vector3 pos, Vector3 dir, Ogre::String name, double _V_MAX)
{
	this->name = name;
	
	sceneManager = sm;
	entity = sm->createEntity(name, "skate.mesh");
	
	node = sm->getRootSceneNode()->createChildSceneNode(name);
	node->setPosition(pos);
	node->attachObject(entity);
	node->setInheritOrientation(true);
	node->setDirection(dir);
	
	dv = 0;
	V_MAX = _V_MAX;

	ang = 0;
	ANG_MAX = 1;

	shoot_time = 0;
	SHOOT_TIME_MAX = 100;
	id = 0;

	life = 100;

	smoker = new PSysFromScript(pos,sm,"Smoke");

}

Skate::~Skate(void)
{
}

void Skate::update(float t, float dt)
{
	if(ang > 0) {
		ang -= 0.1;
	}
	if(ang < 0) {
		ang += 0.1;
	}
	if(ang < 0.1 && ang > -0.1) {
		ang = 0;
	}
	
	double v = actor->getLinearVelocity().magnitude();
	
	if(v > 0 && ang != 0) {
		double ax = actor->getAngularVelocity().x;
		double ay = actor->getAngularVelocity().y;
		double az = actor->getAngularVelocity().z;
		ay = ang*2;
		actor->setAngularVelocity(NxVec3(ax,ay,az));
		
	}
	

	if(shoot_time > 0) {
		shoot_time--;
	}
	
}

void Skate::move(float amount)
{
	
	double v = actor->getLinearVelocity().magnitude();
	if(abs(v) < V_MAX) {
		actor->addLocalForce(NxVec3(0,0,amount*1000), NX_FORCE);
	}
	
}

void Skate::turn(float _ang)
{
	ang=_ang;

	
}

bool Skate::shoot(SceneNode* ballNode) {
	if(shoot_time == 0) {
		
		Entity *ballEnt = sceneManager->createEntity(name+"ball"+StringConverter::toString(id),"strippedBall.mesh");
		ballNode->attachObject(ballEnt);
		ballNode->setScale(0.3,0.3,0.3);
		double ar = node->getOrientation().getYaw().valueRadians();
		ballNode->setPosition(
			node->getPosition().x + sin(ar)*2 +cos(ar),
			node->getPosition().y+1.5,
			node->getPosition().z + cos(ar)*2 - sin(ar));
		
		ballNode->setOrientation(
			node->getOrientation().w,
			node->getOrientation().x,
			node->getOrientation().y,
			node->getOrientation().z
			);
	
		id++;

		shoot_time = SHOOT_TIME_MAX;
		return true;
	}
	return false;
}

Ogre::SceneNode* Skate::getSceneNode() {
	return node;
}

bool Skate::isAlive() {
	return life > 0;
}